Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(1480)

Unified Diff: media/audio/audio_output_dispatcher.h

Issue 9691001: Audio software mixer. (Closed) Base URL: http://src.chromium.org/svn/trunk/src/
Patch Set: Created 8 years, 8 months ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
« no previous file with comments | « media/audio/audio_manager_base.cc ('k') | media/audio/audio_output_dispatcher.cc » ('j') | no next file with comments »
Expand Comments ('e') | Collapse Comments ('c') | Show Comments Hide Comments ('s')
Index: media/audio/audio_output_dispatcher.h
===================================================================
--- media/audio/audio_output_dispatcher.h (revision 132979)
+++ media/audio/audio_output_dispatcher.h (working copy)
@@ -2,33 +2,26 @@
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
// AudioOutputDispatcher is a single-threaded class that dispatches creation and
// deletion of audio output streams. AudioOutputProxy objects use this class to
// allocate and recycle actual audio output streams. When playback is started,
// the proxy calls StreamStarted() to get an output stream that it uses to play
// audio. When playback is stopped, the proxy returns the stream back to the
// dispatcher by calling StreamStopped().
//
-// To avoid opening and closing audio devices more frequently than necessary,
-// each dispatcher has a pool of inactive physical streams. A stream is closed
-// only if it hasn't been used for a certain period of time (specified via the
-// constructor).
-//
-// AudioManagerBase creates one AudioOutputDispatcher on the audio thread for
-// each possible set of audio parameters. I.e streams with different parameters
-// are managed independently. The AudioOutputDispatcher instance is then
-// deleted on the audio thread when the AudioManager shuts down.
+// AudioManagerBase creates one specialization of AudioOutputDispatcher on the
+// audio thread for each possible set of audio parameters. I.e streams with
+// different parameters are managed independently. The AudioOutputDispatcher
+// instance is then deleted on the audio thread when the AudioManager shuts
+// down.
#ifndef MEDIA_AUDIO_AUDIO_OUTPUT_DISPATCHER_H_
#define MEDIA_AUDIO_AUDIO_OUTPUT_DISPATCHER_H_
-#include <vector>
-#include <list>
-
#include "base/basictypes.h"
#include "base/memory/ref_counted.h"
-#include "base/memory/weak_ptr.h"
#include "base/timer.h"
+#include "media/audio/audio_io.h"
#include "media/audio/audio_manager.h"
#include "media/audio/audio_parameters.h"
@@ -36,77 +29,51 @@
namespace media {
-class AudioOutputStream;
+class AudioOutputProxy;
class MEDIA_EXPORT AudioOutputDispatcher
: public base::RefCountedThreadSafe<AudioOutputDispatcher> {
public:
- // |close_delay_ms| specifies delay after the stream is paused until
- // the audio device is closed.
AudioOutputDispatcher(AudioManager* audio_manager,
- const AudioParameters& params,
- base::TimeDelta close_delay);
- ~AudioOutputDispatcher();
+ const AudioParameters& params);
- // Called by AudioOutputProxy when the stream is closed. Opens a new
- // physical stream if there are no pending streams in |idle_streams_|.
+ // Called by AudioOutputProxy to open the stream.
// Returns false, if it fails to open it.
- bool StreamOpened();
+ virtual bool OpenStream() = 0;
- // Called by AudioOutputProxy when the stream is started. If there
- // are pending streams in |idle_streams_| then it returns one of them,
- // otherwise creates a new one. Returns a physical stream that must
- // be used, or NULL if it fails to open audio device. Ownership of
- // the result is passed to the caller.
- AudioOutputStream* StreamStarted();
+ // Called by AudioOutputProxy when the stream is started.
+ // Uses |callback| to get source data and report errors, if any.
+ // Does *not* take ownership of this callback.
+ // Returns true if started successfully, false otherwise.
+ virtual bool StartStream(AudioOutputStream::AudioSourceCallback* callback,
+ AudioOutputProxy* stream_proxy) = 0;
- // Called by AudioOutputProxy when the stream is stopped. Holds the
- // stream temporarily in |pausing_streams_| and then |stream| is
- // added to the pool of pending streams (i.e. |idle_streams_|).
- // Ownership of the |stream| is passed to the dispatcher.
- void StreamStopped(AudioOutputStream* stream);
+ // Called by AudioOutputProxy when the stream is stopped.
+ // Ownership of the |stream_proxy| is passed to the dispatcher.
+ virtual void StopStream(AudioOutputProxy* stream_proxy) = 0;
+
+ // Called by AudioOutputProxy when the volume is set.
+ virtual void StreamVolumeSet(AudioOutputProxy* stream_proxy,
+ double volume) = 0;
+
// Called by AudioOutputProxy when the stream is closed.
- void StreamClosed();
+ virtual void CloseStream(AudioOutputProxy* stream_proxy) = 0;
// Called on the audio thread when the AudioManager is shutting down.
- void Shutdown();
+ virtual void Shutdown() = 0;
- private:
- friend class AudioOutputProxyTest;
+ protected:
+ friend class base::RefCountedThreadSafe<AudioOutputDispatcher>;
+ virtual ~AudioOutputDispatcher();
- // Creates a new physical output stream, opens it and pushes to
- // |idle_streams_|. Returns false if the stream couldn't be created or
- // opened.
- bool CreateAndOpenStream();
-
- // A task scheduled by StreamStarted(). Opens a new stream and puts
- // it in |idle_streams_|.
- void OpenTask();
-
- // Before a stream is reused, it should sit idle for a bit. This task is
- // called once that time has elapsed.
- void StopStreamTask();
-
- // Called by |close_timer_|. Closes all pending stream.
- void ClosePendingStreams();
-
// A no-reference-held pointer (we don't want circular references) back to the
// AudioManager that owns this object.
AudioManager* audio_manager_;
MessageLoop* message_loop_;
AudioParameters params_;
- base::TimeDelta pause_delay_;
- size_t paused_proxies_;
- typedef std::list<AudioOutputStream*> AudioOutputStreamList;
- AudioOutputStreamList idle_streams_;
- AudioOutputStreamList pausing_streams_;
-
- // Used to post delayed tasks to ourselves that we cancel inside Shutdown().
- base::WeakPtrFactory<AudioOutputDispatcher> weak_this_;
- base::DelayTimer<AudioOutputDispatcher> close_timer_;
-
+ private:
DISALLOW_COPY_AND_ASSIGN(AudioOutputDispatcher);
};
« no previous file with comments | « media/audio/audio_manager_base.cc ('k') | media/audio/audio_output_dispatcher.cc » ('j') | no next file with comments »
@@ -2,33 +2,26 @@
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.
-// AudioOutputDispatcher is a single-threaded class that dispatches creation and
-// deletion of audio output streams. AudioOutputProxy objects use this class to
-// allocate and recycle actual audio output streams. When playback is started,
-// the proxy calls StreamStarted() to get an output stream that it uses to play
-// audio. When playback is stopped, the proxy returns the stream back to the
-// dispatcher by calling StreamStopped().
+// AudioOutputDispatcher is a single-threaded base class that dispatches
+// creation and deletion of audio output streams. AudioOutputProxy objects use
+// this class to allocate and recycle actual audio output streams. When playback
+// is started, the proxy calls StartStream() to get an output stream that it
+// uses to play audio. When playback is stopped, the proxy returns the stream
+// back to the dispatcher by calling StopStream().
//

Powered by Google App Engine
This is Rietveld 408576698